import { FileStat, FileSystemProvider as CSSFileSystemProvider, FileType } from 'vscode-css-languageservice';
export declare class FileSystemProvider implements CSSFileSystemProvider {
    private promisifyStat;
    private promisifyReaddir;
    constructor();
    stat(uri: string): Promise<FileStat>;
    private unknownStat;
    private getFileType;
    readDirectory(uri: string): Promise<Array<[string, FileType]>>;
}
